package common

import (
	log "asapo_common/logger"
	"errors"
	"os"
	"strings"
)

type ReceiverInfo struct {
	StaticEndpoints []string
	MaxConnections  int
	UseIBAddress    bool
}

type MonitoringInfo struct {
	StaticEndpoint string
}

type BrokerInfo struct {
	StaticEndpoint string
}

type MongoInfo struct {
	StaticEndpoint string
}

type FtsInfo struct {
	StaticEndpoint string
}

type Settings struct {
	Receiver            ReceiverInfo
	Broker              BrokerInfo
	Monitoring          MonitoringInfo
	Mongo               MongoInfo
	FileTransferService FtsInfo
	ConsulEndpoints     []string
	Mode                string
	Kubernetes          struct {
		Mode       string
		ConfigFile string
		Namespace  string
	}
	Port     int
	LogLevel string
}

func (settings *Settings) setCurrentServer() error {
	hostname, err := os.Hostname()
	if err != nil {
		return err
	}

	if settings.Receiver.StaticEndpoints[0] > "0.0.0.0" {
		settings.Receiver.StaticEndpoints[0] = strings.Replace(settings.Receiver.StaticEndpoints[0], "0.0.0.0", hostname, 1)
		log.Info("Replace Receiver server to " + hostname)
	}
	if settings.Broker.StaticEndpoint > "0.0.0.0" {
		settings.Broker.StaticEndpoint = strings.Replace(settings.Broker.StaticEndpoint, "0.0.0.0", hostname, 1)
		log.Info("Replace Broker server to " + hostname)
	}
	if settings.Monitoring.StaticEndpoint > "0.0.0.0" {
		settings.Monitoring.StaticEndpoint = strings.Replace(settings.Monitoring.StaticEndpoint, "0.0.0.0", hostname, 1)
		log.Info("Replace Monitoring server to " + hostname)
	}
	if settings.Mongo.StaticEndpoint > "0.0.0.0" {
		settings.Mongo.StaticEndpoint = strings.Replace(settings.Mongo.StaticEndpoint, "0.0.0.0", hostname, 1)
		log.Info("Replace Mongo server to " + hostname)
	}
	if settings.FileTransferService.StaticEndpoint > "0.0.0.0" {
		settings.FileTransferService.StaticEndpoint = strings.Replace(settings.FileTransferService.StaticEndpoint, "0.0.0.0", hostname, 1)
		log.Info("Replace FileTransferService server to " + hostname)
	}
	return nil
}

func (settings *Settings) Validate() error {
	if settings.Mode == "static" {
		if len(settings.Receiver.StaticEndpoints) == 0 || len(settings.Broker.StaticEndpoint) == 0 || len(settings.Mongo.StaticEndpoint) == 0 {
			return errors.New("static endpoints not set")
		}
		err := settings.setCurrentServer()
		if err != nil {
			return errors.New("unable to get server name")
		}
	}

	if settings.Receiver.MaxConnections == 0 {
		return errors.New("Max connections not set")
	}

	if settings.Port == 0 {
		return errors.New("Server port not set")
	}

	if settings.Mode == "" {
		return errors.New("Mode not set")
	}

	if settings.Mode != "static" && settings.Mode != "consul" && settings.Mode != "kubernetes" {
		return errors.New("wrong mode: " + settings.Mode + ", (allowed static|consul|kubernetes)")
	}

	return nil
}
