import {reactive} from "vue";
import {GroupDelayResponse} from "../generated_proto/AsapoMonitoringQueryService_pb";
import {FixedTimeRange} from "../lib/TimeRange";
import {timeStore} from "./timeStore";

export interface GroupDelayStoreState {
    lastUpdateRange: FixedTimeRange,
    GroupIdList: string[];
}

class GroupDelayStore {
    private internalState: GroupDelayStoreState = reactive({
        lastUpdateRange: {
            fromUnixSec: 0,
            toUnixSec: 0
        },
        GroupIdList: [],
    });

    public get state(): Readonly<GroupDelayStoreState> {
        return this.internalState;
    }

    private beautifyDelayMs(delayMs: number): string {
        if (delayMs > 60 * 60 * 1000) {
            return ">1H"
        } else if (delayMs > 60 * 1000) {
            return Math.floor(delayMs / 60000).toString() + " mins"
        } else if (delayMs > 1000) {
            return Math.floor(delayMs / 1000).toString() + " secs"
        }
        return "0." + delayMs.toString().padStart(3, '0') + " secs"
    }

    public parseServerResponse(groupDelayData: GroupDelayResponse): void {
        const newGroupIds: string[] = [];
        for (const groupId of groupDelayData.getGroupidsList()) {
            const value: string = groupId.getGroupid() + groupId.getDelayms()
            newGroupIds.push(groupId.getGroupid().padEnd(25, '\xa0') +
                groupId.getPipelinestep().padEnd(25, '\xa0') +
                this.beautifyDelayMs(groupId.getDelayms()))
        }

        this.internalState.GroupIdList = newGroupIds

        this.internalState.lastUpdateRange.fromUnixSec = timeStore.state.lastFixedTimeRange.fromUnixSec;
        this.internalState.lastUpdateRange.toUnixSec = timeStore.state.lastFixedTimeRange.toUnixSec;
    }

    public couldBeOutdated(): boolean {
        const maxDelayOldnessInSec = 5;

        return Math.abs(this.internalState.lastUpdateRange.fromUnixSec - timeStore.state.lastFixedTimeRange.fromUnixSec) > maxDelayOldnessInSec ||
            Math.abs(this.internalState.lastUpdateRange.toUnixSec - timeStore.state.lastFixedTimeRange.toUnixSec) > maxDelayOldnessInSec
    }
}

export const groupDelayStore = new GroupDelayStore();
